@@ -34,7 +34,9 @@ use strict;
 use warnings;
 
 use POSIX qw(:sys_wait_h);
+use File::Basename;
 use File::Temp;
+use File::Copy;
 use MIME::Base64;
 
 use Dpkg::ErrorHandling;
@@ -46,7 +48,7 @@ use Dpkg::OpenPGP::ErrorCodes;
 use parent qw(Dpkg::OpenPGP::Backend);
 
 sub DEFAULT_CMDV {
-    return [ qw(gpgv) ];
+    return [ qw(gpgv-sq gpgv) ];
 }
 
 sub DEFAULT_CMDSTORE {
@@ -54,7 +56,7 @@ sub DEFAULT_CMDSTORE {
 }
 
 sub DEFAULT_CMD {
-    return [ qw(gpg) ];
+    return [ qw(gpg-sq gpg) ];
 }
 
 sub has_backend_cmd {
@@ -296,6 +298,18 @@ sub inline_sign {
 
     return OPENPGP_MISSING_CMD if ! $self->has_backend_cmd();
 
+    my $file = basename($data);
+    my $signdir = File::Temp->newdir('dpkg-sign.XXXXXXXX', TMPDIR => 1);
+    my $signfile = "$signdir/$file";
+
+    # Make sure the file to sign ends with a newline, as GnuPG does not adhere
+    # to the OpenPGP specification (see <https://dev.gnupg.org/T7106>).
+    copy($data, $signfile);
+    open my $signfh, '>>', $signfile
+        or syserr(g_('cannot open %s'), $signfile);
+    print { $signfh } "\n";
+    close $signfh or syserr(g_('cannot close %s'), $signfile);
+
     my @exec = ($self->{cmd});
     push @exec, _gpg_options_weak_digests();
     push @exec, qw(--utf8-strings --textmode --armor);
